/*
 *  TOPPERS ECHONET Lite Communication Middleware
 * 
 *  Copyright (C) 2014-2018 Cores Co., Ltd. Japan
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id$
 */

/* 
 *  サンプルプログラム(1)の本体
 */

#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include <sil.h>
#include <string.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "echonet_main.h"
#include "echonet_cfg.h"
#include "rx630.h"

/* TODO: メーカーコードを設定 */
#define MAKER_CODE	{ 0x00, 0x00, 0xB3 }	/* TOPPERSプロジェクト */

/* ノードプロファイルオブジェクト */
struct ecn_cls0EF0_t node_profile_data = {
	0x30,	/* 動作状態 */
	{ 0x01, 0x0A, { 0x01, 0x00 } },	/* Ｖｅｒｓｉｏｎ情報 */
	{
		0xFE,	/* 下位通信層IDフィールド */
		{ MAKER_CODE },	/* メーカーコード */
		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, },	/* ユニークID部(メーカー独自) */
	},
	0x0000,	/* 異常内容 */
	{ MAKER_CODE },	/* メーカーコード */
};

/* 温度センサクラス */
struct ecn_cls0011_t temperature_sensor1_data = {
	0x30,	/* 動作状態 */
	0xF554,	/* 温度計測値 */
	0x00,	/* 設置場所 */
	{ 0x00, 0x00, 'C', 0x00 },	/* 規格Ｖｅｒｓｉｏｎ情報 */
	0x42,	/* 異常発生状態 */
	{ MAKER_CODE },	/* メーカーコード */
};

/* 温度センサクラス */
struct ecn_cls0011_t temperature_sensor2_data = {
	0x30,	/* 動作状態 */
	0xF554,	/* 温度計測値 */
	0x00,	/* 設置場所 */
	{ 0x00, 0x00, 'C', 0x00 },	/* 規格Ｖｅｒｓｉｏｎ情報 */
	0x42,	/* 異常発生状態 */
	{ MAKER_CODE },	/* メーカーコード */
};

/* 湿度センサクラス */
struct ecn_cls0012_t humidity_sensor_data = {
	0x30,	/* 動作状態 */
	0x00,	/* 相対湿度計測値 */
	0x00,	/* 設置場所 */
	{ 0x00, 0x00, 'C', 0x00 },	/* 規格Ｖｅｒｓｉｏｎ情報 */
	0x42,	/* 異常発生状態 */
	{ MAKER_CODE },	/* メーカーコード */
};

/*
 * 動作状態ON/OFF設定関数（0x30, 0x31のみ受け付け）
 */
int onoff_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)
{
	ER ret;
	uint8_t data[2];

	/* サイズが１以外は受け付けない */
	if(size != 1)
		return 0;

	*anno = *((uint8_t *)item->exinf) != *((uint8_t *)src);

	switch(*(uint8_t *)src){
	/* ONの場合 */
	case 0x30:
	/* OFFの場合 */
	case 0x31:
		*((uint8_t *)item->exinf) = *((uint8_t *)src);
		/* メインタスクに通知 */
		data[0] = 0x80;
		data[1] = *(uint8_t *)src;
		ret = ecn_brk_wai(data, sizeof(data));
		if(ret != E_OK){
			syslog(LOG_ERROR, "ecn_brk_wai");
			return 1;
		}
		break;
	/* 0x30か0x31以外は受け付けない */
	default:
		return 0;
	}

	return 1;
}

/*
 * 異常発生状態設定関数（0x41, 0x42のみ受け付け）
 */
int alarm_prop_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)
{
	/* サイズが１以外は受け付けない */
	if(size != 1)
		return 0;

	*anno = *((uint8_t *)item->exinf) != *((uint8_t *)src);

	switch(*(uint8_t *)src){
	/* 異常発生ありの場合 */
	case 0x41:
	/* 異常発生なしの場合 */
	case 0x42:
		*((uint8_t *)item->exinf) = *((uint8_t *)src);
		break;
	/* 0x41か0x42以外は受け付けない */
	default:
		return 0;
	}

	return 1;
}

/*
 * 温度測定値設定関数
 */
int temperature_set(const EPRPINIB *item, const void *src, int size, bool_t *anno)
{
	struct ecn_cls0011_t *eobj = (struct ecn_cls0011_t *)item->exinf;
	int16_t value;
	ER ret;
	uint8_t data[1];

	/* サイズが2以外は受け付けない */
	if(size != 2)
		return 0;

	value = (((uint8_t *)src)[0] << 8) | (((uint8_t *)src)[1] & 0xFF);

	if((value >= -2732) && (value <= 32766)){
		eobj->propertyE0 = value;
		/* メインタスクに通知 */
		data[0] = 0xE0;
		ret = ecn_brk_wai(data, sizeof(data));
		if(ret != E_OK){
			syslog(LOG_ERROR, "ecn_brk_wai");
			return 2;
		}
	}
	/* 上記以外は受け付けない */
	else{
		return 0;
	}

	return 2;
}

/* 
 * 温度計測値取得関数
 */
int temperature_get(const EPRPINIB *item, void *dst, int size)
{
	struct ecn_cls0011_t *eobj = (struct ecn_cls0011_t *)item->exinf;
	int16_t value = eobj->propertyE0;

	/* サイズが2以外は受け付けない */
	if(size != 2)
		return 0;

	((uint8_t *)dst)[0] = value >> 8;
	((uint8_t *)dst)[1] = value & 0xFF;

	return 2;
}

static void main_initialize();
static int main_get_timer();
static void main_progress(int interval);
static void main_recv_esv(T_EDATA *esv);
static void main_break_wait(uint8_t *brkdat, int32_t len);
static void main_timeout();

/*
 *  メインタスク
 */
void echonet_main_task(intptr_t exinf)
{
	ER ret, ret2;
	SYSTIM prev, now;
	int timer;
	T_EDATA *esv;
	uint8_t brkdat[64];
	int32_t len;

	/* アプリケーションの初期化 */
	main_initialize();

	ret2 = get_tim(&now);
	if (ret2 != E_OK){
		syslog(LOG_ERROR, "get_tim");
		return;
	}

	for(;;){
		prev = now;

		/* タイマー取得 */
		timer = main_get_timer();

		/* 応答電文待ち */
		ret = ecn_trcv_esv(&esv, timer);
		if ((ret != E_OK) && (ret != E_BRK) && (ret != E_TMOUT)){
			syslog(LOG_ERROR, "ecn_trcv_esv");
			break;
		}

		ret2 = get_tim(&now);
		if (ret2 != E_OK){
			syslog(LOG_ERROR, "get_tim");
			break;
		}

		/* 時間経過 */
		main_progress(now - prev);

		/* Echonet電文受信の場合 */
		if (ret == E_OK) {
			/* Echonet電文受信処理 */
			main_recv_esv(esv);

			/* 領域解放 */
			ret = ecn_rel_esv(esv);
			if (ret != E_OK){
				syslog(LOG_ERROR, "ecn_rel_esv");
				break;
			}
		}
		/* 応答電文待ちの割り込みの場合 */
		else if (ret == E_BRK) {
			/* 応答電文待ちの割り込みデータ取得 */
			ret = ecn_get_brk_dat(esv, brkdat, sizeof(brkdat), &len);
			if (ret != E_OK){
				syslog(LOG_ERROR, "ecn_get_brk_dat");
				break;
			}

			/* 応答電文待ちの割り込み処理 */
			main_break_wait(brkdat, len);

			/* 領域解放 */
			ret = ecn_rel_esv(esv);
			if (ret != E_OK){
				syslog(LOG_ERROR, "ecn_rel_esv");
				break;
			}
		}

		/* タイムアウト処理 */
		main_timeout();
	}
}

bool_t started = false;

void echonet_change_netif_link(uint8_t link_up, uint8_t up)
{
	ER ret;

	if (link_up == 0)
		return;

	if (up && !started) {
		started = true;

		/* ECHONETミドルウェアを起動 */
		ret = ecn_sta_svc();
		if (ret != E_OK)
			return;
	}

	/* メインタスクに通知 */
	uint8_t data[2];
	data[0] = 0x01;
	data[1] = up ? 0x01 : 0x02;
	ret = ecn_brk_wai(data, sizeof(data));
	if (ret != E_OK) {
		syslog(LOG_ERROR, "ecn_brk_wai");
		return;
	}
}

enum main_state_t{
	main_state_idle,
};

int main_timer = TMO_FEVR;
enum main_state_t main_state = main_state_idle;

enum main_led_state_t{
	main_led_state_idle,
	main_led_state_on_o,
	main_led_state_on_n,
	main_led_state_off_o,
	main_led_state_off_f1,
	main_led_state_off_f2,
	main_led_state_space1,
	main_led_state_tmpr_minus,
	main_led_state_tmpr_100,
	main_led_state_tmpr_10,
	main_led_state_tmpr_1,
	main_led_state_space2,
	main_led_state_humid_10,
	main_led_state_humid_1,
	main_led_state_space3,
};

int main_led_timer = TMO_FEVR;
enum main_led_state_t main_led_state = main_led_state_idle;
bool_t main_led_update;

bool_t main_btn1_state;
int main_btn1_count = 0;
bool_t main_btn2_state;
int main_btn2_count = 0;

/*
 * 初期化
 */
static void main_initialize()
{
	uint8_t btn1, btn2;

	/*
	 * モジュールストップ機能の設定(S12AD)
	 */
	sil_wrh_mem((uint16_t *)SYSTEM_PRCR_ADDR, (uint16_t)0xA502);	/* 書込み許可 */
	sil_wrw_mem((uint32_t *)SYSTEM_MSTPCRA_ADDR,
		sil_rew_mem((uint32_t *)SYSTEM_MSTPCRA_ADDR) & ~SYSTEM_MSTPCRA_MSTPA17_BIT);
	sil_wrh_mem((uint16_t *)SYSTEM_PRCR_ADDR, (uint16_t)0xA500);	/* 書込み禁止 */

	/* LEDを"0000"と表示 */
	sil_wrb_mem((uint8_t *)0x0008C02A, 0x00);

	/* 7segを"0"と表示 */
	sil_wrb_mem((uint8_t *)0x0008C02D, 0xC0);

	/* 10ms後にボタン状態を確認 */
	main_timer = 10 * 1000;

	/* 12bitADC初期化 */
	sil_wrh_mem((uint16_t *)S12AD_ADEXICR_ADDR, 0x0000);	/* 温度センサ出力、内部基準電圧非選択 */
	sil_wrh_mem((uint16_t *)S12AD_ADANS0_ADDR, 0x0001);		/* 変換ポートAN000選択、AN001～AN015非選択 */
	sil_wrh_mem((uint16_t *)S12AD_ADANS1_ADDR, 0x0000);		/* 変換ポートAN016～AN020非選択 */

	if(temperature_sensor1_data.property80 == 0x30)
		main_led_state = main_led_state_on_o;
	else
		main_led_state = main_led_state_off_o;
	main_led_timer = 0;

	/* ボタン状態読み込み */
	btn1 = sil_reb_mem((uint8_t *)0x0008C040);
	btn2 = sil_reb_mem((uint8_t *)0x0008C040);
	main_btn1_state = (btn1 & 0x20) != 0;
	main_btn2_state = (btn2 & 0x80) != 0;
}

/*
 * タイマー取得
 */
static int main_get_timer()
{
	int result = main_timer;

	if((result == TMO_FEVR)
		|| ((main_led_timer != TMO_FEVR) && (main_led_timer < result))){
		result = main_led_timer;
	}

	return result;
}

/*
 * 時間経過
 */
static void main_progress(int interval)
{
	if(main_timer != TMO_FEVR){
		main_timer -= interval;
		if(main_timer < 0){
			main_timer = 0;
		}
	}

	if(main_led_timer != TMO_FEVR){
		main_led_timer -= interval;
		if(main_led_timer < 0){
			main_led_timer = 0;
		}
	}
}

static bool_t check_response(uint8_t pdc, uint8_t *p_edt);

/*
 * Echonet電文受信処理
 */
static void main_recv_esv(T_EDATA *esv)
{
	ER ret;
	uint8_t epc;
	uint8_t pdc;
	uint8_t p_edt[256];
	T_ENUM_EPC enm;
	bool_t in_get = false;

	ret = ecn_itr_ini(&enm, esv);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_itr_ini");
		return;
	}

	for(;;) {
		while((ret = ecn_itr_nxt(&enm, &epc, &pdc, p_edt)) == E_OK) {
			switch (epc) {
			case 0xE0:
				if(in_get && (esv->hdr.edata.esv == ESV_SET_GET_RES)){
					check_response(pdc, p_edt);
				}
				break;
			}
		}
		if(ret != E_BOVR){
			syslog(LOG_ERROR, "ecn_itr_nxt");
			break;
		}
		if(enm.is_eof)
			break;

		/* GET_SETのGETへ */
		in_get = true;
	}
}

/*
 * 応答電文待ちの割り込み処理
 */
static void main_break_wait(uint8_t *brkdat, int32_t len)
{
	if (brkdat[0] == 0x01) {
		// Link up/down
		return;
	}

	if ((len == 2) && (brkdat[0] == 0x80)) {
		switch(brkdat[1]){
		/* ONの場合 */
		case 0x30:
			/* LEDの"."をON */
			sil_wrb_mem((uint8_t *)0x0008C02A, sil_reb_mem((uint8_t *)0x0008C02A) | 0x40);
			break;
		/* OFFの場合 */
		case 0x31:
			/* LEDの"."をOFF */
			sil_wrb_mem((uint8_t *)0x0008C02A, sil_reb_mem((uint8_t *)0x0008C02A) & ~0x40);
			break;
		}
	}

	switch(main_led_state){
	case main_led_state_idle:
		if(temperature_sensor1_data.property80 == 0x30)
			main_led_state = main_led_state_on_o;
		else
			main_led_state = main_led_state_off_o;
		main_led_timer = 0;
		break;
	default:
		main_led_update = true;
		break;
	}
}

static void main_ontimer();
static void main_led_ontimer();

/*
 * タイムアウト処理
 */
static void main_timeout()
{
	if(main_timer == 0){
		main_ontimer();
	}

	if(main_led_timer == 0){
		main_led_ontimer();
	}
}

static int16_t main_ad_tempr = 200;
static uint16_t main_ad_table[64];
#define main_ad_table_count (sizeof(main_ad_table) / sizeof(main_ad_table[0]))
static int main_ad_pos = 0;
static uint32_t main_ad_avelage;
static int main_count;

static void main_ad_change();
static void main_btn1_change(bool_t push);
static void main_btn2_change(bool_t push);

uint8_t led_disp[] = { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07 };

static void main_ontimer()
{
	uint8_t btn1, btn2;
	uint16_t ad_value;
	int16_t ad_tempr;
	uint8_t sseg = sil_reb_mem((uint8_t *)0x0008C02A) & ~0x07;

	switch(main_state){
	case main_state_idle:
		/* 10ms後にボタン状態を確認 */
		main_timer = 10 * 1000;

		/* ADCの変換結果取得 */
		if((sil_reb_mem((uint8_t *)S12AD_ADCSR_ADDR) & S12AD_ADCSR_ADST_BIT) == 0){
			main_ad_avelage -= main_ad_table[main_ad_pos];
			ad_value = sil_reh_mem((uint16_t *)S12AD_ADDR0_ADDR);
			main_ad_table[main_ad_pos] = ad_value;
			main_ad_avelage += ad_value;

			main_ad_pos++;
			if(main_ad_pos >= main_ad_table_count){
				main_ad_pos = 0;
			}

			ad_tempr = ((int16_t)(((main_ad_avelage / main_ad_table_count) >> 3) & 0x1FF)) - 100;
			if(main_ad_tempr != ad_tempr){
				/* １秒変化が続いたら */
				if(main_count < 100){
					main_count++;
				}
				else{
					main_ad_tempr = ad_tempr;

					/* ADC状態変化処理 */
					main_ad_change();
				}
			}
			else{
				main_count = 0;
			}

			/* LEDに温度計測値を表示 */
			sil_wrb_mem((uint8_t *)0x0008C02A, sseg | led_disp[(main_ad_avelage / main_ad_table_count) >> 9]);

			/* 変換開始（シングルスキャンモード） */
			sil_wrb_mem((uint8_t *)S12AD_ADCSR_ADDR, S12AD_ADCSR_ADST_BIT);
		}

		/* ボタン状態読み込み */
		btn1 = sil_reb_mem((uint8_t *)0x0008C040);
		btn2 = sil_reb_mem((uint8_t *)0x0008C040);

		/* ボタン１の処理 */
		if(((btn1 & 0x20) != 0) && !main_btn1_state){
			main_btn1_count++;
			if(main_btn1_count > 10){
				main_btn1_count = 0;
				main_btn1_state = true;

				main_btn1_change(true);
			}
		}
		else if(((btn1 & 0x20) == 0) && main_btn1_state){
			main_btn1_count++;
			if(main_btn1_count > 10){
				main_btn1_count = 0;
				main_btn1_state = false;

				main_btn1_change(false);
			}
		}

		/* ボタン２の処理 */
		if(((btn2 & 0x80) != 0) && !main_btn2_state){
			main_btn2_count++;
			if(main_btn2_count > 10){
				main_btn2_count = 0;
				main_btn2_state = true;

				main_btn2_change(true);
			}
		}
		else if(((btn2 & 0x80) == 0) && main_btn2_state){
			main_btn2_count++;
			if(main_btn2_count > 10){
				main_btn2_count = 0;
				main_btn2_state = false;

				main_btn2_change(false);
			}
		}
		break;
	}
}

uint8_t led_num[] = { 0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0x80, 0x90 };
uint8_t led_onf[] = { 0xA3, 0xAB, 0x8E };

static void main_led_ontimer()
{
	int temp;

	switch(main_led_state){
	case main_led_state_idle:
		/* 7segに' 'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, 0xFF);
		main_led_timer = TMO_FEVR;
		return;
	case main_led_state_on_o:
		/* 7segに'o'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_onf[0]);
		main_led_state = main_led_state_on_n;
		break;
	case main_led_state_on_n:
		/* 7segに'n'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_onf[1]);
		main_led_state = main_led_state_space1;
		break;
	case main_led_state_off_o:
		/* 7segに'o'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_onf[0]);
		main_led_state = main_led_state_off_f1;
		break;
	case main_led_state_off_f1:
		/* 7segに'f'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_onf[2]);
		main_led_state = main_led_state_off_f2;
		break;
	case main_led_state_off_f2:
		/* 7segに'f'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_onf[2]);
		main_led_state = main_led_state_space1;
		break;
	case main_led_state_space1:
		/* 7segに' 'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, 0xFF);
		if(temperature_sensor1_data.propertyE0 >= 0){
			main_led_state = main_led_state_tmpr_10;
		}
		else{
			main_led_state = main_led_state_tmpr_minus;
		}
		break;
	case main_led_state_tmpr_minus:
		/* 7segに'-'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, 0xBF);
		if((temperature_sensor1_data.propertyE0 >= 100)
			|| (temperature_sensor1_data.propertyE0 <= -100))
			main_led_state = main_led_state_tmpr_100;
		else
			main_led_state = main_led_state_tmpr_10;
		break;
	case main_led_state_tmpr_100:
		temp = (temperature_sensor1_data.propertyE0 / 1000) % 10;
		if(temp < 0)
			temp = -temp;
		/* 7segに温度計測値10の位を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_num[temp]);
		main_led_state = main_led_state_tmpr_10;
		break;
	case main_led_state_tmpr_10:
		temp = (temperature_sensor1_data.propertyE0 / 100) % 10;
		if(temp < 0)
			temp = -temp;
		/* 7segに温度計測値10の位を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_num[temp]);
		main_led_state = main_led_state_tmpr_1;
		break;
	case main_led_state_tmpr_1:
		temp = (temperature_sensor1_data.propertyE0 / 10) % 10;
		if(temp < 0)
			temp = -temp;
		/* 7segに温度計測値10の位を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_num[temp] & (~0x80));
		main_led_state = main_led_state_space2;
		break;
	case main_led_state_space2:
		/* 7segに' 'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, 0xFF);
		main_led_state = main_led_state_humid_10;
		break;
	case main_led_state_humid_10:
		temp = humidity_sensor_data.propertyE0 / 10;
		/* 7segに湿度計測値10の位を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_num[temp]);
		main_led_state = main_led_state_humid_1;
		break;
	case main_led_state_humid_1:
		temp = humidity_sensor_data.propertyE0 % 10;
		/* 7segに湿度計測値10の位を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, led_num[temp] & (~0x80));
		main_led_state = main_led_state_space3;
		break;
	case main_led_state_space3:
		/* 7segに' 'を表示 */
		if(temperature_sensor1_data.property80 == 0x30)
			/* 7segに'.'を表示 */
			sil_wrb_mem((uint8_t *)0x0008C02D, 0x7F);
		else
			/* 7segに' 'を表示 */
			sil_wrb_mem((uint8_t *)0x0008C02D, 0xFF);

		if(main_led_update){
			main_led_update = false;

			if(temperature_sensor1_data.property80 == 0x30)
				main_led_state = main_led_state_on_o;
			else
				main_led_state = main_led_state_off_o;
		}
		else{
			main_led_state = main_led_state_idle;
			main_led_timer = TMO_FEVR;
			return;
		}
		break;
	default:
		/* 7segに' 'を表示 */
		sil_wrb_mem((uint8_t *)0x0008C02D, 0xFF);
		main_led_state = main_led_state_idle;
		main_led_timer = TMO_FEVR;
		return;
	}

	main_led_timer = 500 * 1000;
}

/*
 * ADC状態変化処理
 */
static void main_ad_change()
{
	ER ret;
	T_EDATA *esv;
	uint8_t p_edt[2];

	p_edt[0] = ((uint16_t)main_ad_tempr) >> 8;
	p_edt[1] = ((uint16_t)main_ad_tempr) & 0xFF;

	/* プロパティ設定電文作成 */
	ret = ecn_esv_setc(&esv, TEMP_SENSOR_01_EOBJ, 0xE0, 2, p_edt);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_esv_setc");
		return;
	}

	/* 電文送信 */
	ecn_snd_esv(esv);
}

int16_t main_tmpr = 200;	/* 20.0℃ */

/*
 * ボタン１状態変化処理
 */
static void main_btn1_change(bool_t push)
{
	ER ret;
	T_EDATA *esv;
	uint8_t p_edt[2];
	int trn_pos;

	/* 押されて戻った時に処理する */
	if(push)
		return;

	/* 温度計測値 */
	main_tmpr -= 10;	/* - 1.0℃ */
	if(main_tmpr < -100)	/* -10.0℃までとする */
		main_tmpr = -100;

	p_edt[0] = ((uint16_t)main_tmpr) >> 8;
	p_edt[1] = ((uint16_t)main_tmpr) & 0xFF;

	/* プロパティ値書き込み・読み出し要求電文作成 */
	ret = ecn_esv_set_get(&esv, TEMP_SENSOR_02_EOBJ, 0xE0, 2, p_edt);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_esv_setc");
		return;
	}

	p_edt[0] = 0x30;

	/* 書き込みプロパティ追加 */
	ret = ecn_add_edt(esv, 0x80, 1, p_edt);
	if(ret) {
		syslog(LOG_ERROR, "ecn_add_edt");
		goto error;
	}

	/* プロパティ値書き込み・読み出し要求電文作成：折り返し指定 */
	ret = ecn_trn_set_get(esv, &trn_pos);
	if(ret) {
		syslog(LOG_ERROR, "ecn_trn_set_get");
		goto error;
	}

	/* 読み出しプロパティ追加 */
	ret = ecn_add_epc(esv, 0xE0);
	if(ret) {
		syslog(LOG_ERROR, "ecn_add_epc");
		goto error;
	}

	/* プロパティ値書き込み・読み出し要求電文作成：終了指定 */
	ret = ecn_end_set_get(esv, trn_pos);
	if(ret) {
		syslog(LOG_ERROR, "ecn_end_set_get");
		goto error;
	}

	/* 電文送信 */
	ret = ecn_snd_esv(esv);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_snd_esv");
	}
	return;
error:
	/* 領域解放 */
	ret = ecn_rel_esv(esv);
	if (ret != E_OK){
		syslog(LOG_ERROR, "ecn_rel_esv");
	}
}

/*
 * ボタン２状態変化処理
 */
static void main_btn2_change(bool_t push)
{
	ER ret;
	T_EDATA *esv;
	uint8_t p_edt[2];
	int trn_pos;

	/* 押されて戻った時に処理する */
	if(push)
		return;

	/* 温度計測値 */
	main_tmpr += 10;	/* + 1.0℃ */
	if(main_tmpr > 400)	/* +40.0℃までとする */
		main_tmpr = 400;

	p_edt[0] = ((uint16_t)main_tmpr) >> 8;
	p_edt[1] = ((uint16_t)main_tmpr) & 0xFF;

	/* プロパティ値書き込み・読み出し要求電文作成 */
	ret = ecn_esv_set_get(&esv, TEMP_SENSOR_02_EOBJ, 0xE0, 2, p_edt);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_esv_setc");
		return;
	}

	p_edt[0] = 0x30;

	/* 書き込みプロパティ追加 */
	ret = ecn_add_edt(esv, 0x80, 1, p_edt);
	if(ret) {
		syslog(LOG_ERROR, "ecn_add_edt");
		goto error;
	}

	/* プロパティ値書き込み・読み出し要求電文作成：折り返し指定 */
	ret = ecn_trn_set_get(esv, &trn_pos);
	if(ret) {
		syslog(LOG_ERROR, "ecn_trn_set_get");
		goto error;
	}

	/* 読み出しプロパティ追加 */
	ret = ecn_add_epc(esv, 0xE0);
	if(ret) {
		syslog(LOG_ERROR, "ecn_add_epc");
		goto error;
	}

	/* プロパティ値書き込み・読み出し要求電文作成：終了指定 */
	ret = ecn_end_set_get(esv, trn_pos);
	if(ret) {
		syslog(LOG_ERROR, "ecn_end_set_get");
		goto error;
	}

	/* 電文送信 */
	ret = ecn_snd_esv(esv);
	if(ret != E_OK){
		syslog(LOG_ERROR, "ecn_snd_esv");
	}
	return;
error:
	/* 領域解放 */
	ret = ecn_rel_esv(esv);
	if (ret != E_OK){
		syslog(LOG_ERROR, "ecn_rel_esv");
	}
}

/*
 * ボタンによる温度設定結果確認
 */
static bool_t check_response(uint8_t pdc, uint8_t *p_edt)
{
	if((pdc != 2)
		|| (p_edt[0] != ((uint16_t)main_tmpr) >> 8)
		|| (p_edt[1] != ((uint16_t)main_tmpr) & 0xFF))
	{
		syslog(LOG_WARNING, "epc_set_get response NG");
		return false;
	}
	return true;
}
